#!/usr/bin/python
"""pydicfunc.cgi

  Author: tgz <tgz@users.sourceforge.jp>
  Version: 1.3
  Last Modified: 2004/01/19

  This program is based on GPL.

"""
import cgi
import re, string
import ndtplib


class NDTPSearcher:
    """ super """
    def __init__(self, server):
        self.server = server
        self.NDTP = ndtplib.NDTP(self.server)
        self.version, self.dictdict = self.NDTP.version(), self.NDTP.listdic()
        self.NDTP.quit()
        self.word = ''
        self.result = {}

    def search(self, word):
        self.word = word
        self.NDTP.connect(self.server)
        for num in self.dictdict.keys():
            self.NDTP.setdic(num)
            self.result[num] = self.NDTP.ask(self.word)
        self.NDTP.quit()

    def show(self):
        self.show_header(self.word)
        self.show_main()
        self.show_footer()

    def show_header(self, title):
        print 'Content-type: text/html'
        print ''
        print '<html>'
        print '<head>'
        if self.word != None:
            print '<title>Result of \"%s\"</title>' % title
        print '</head>'
        print '<body>'
        print '<h1>Result of \"%s\"</h1>' % title
        print '<hr>'

    def show_main(self):
        dicts = self.result.keys()
        dicts.sort()
        for num in dicts:
            print '<h2>%s [%d hits]</h2>' % (self.dictdict[num], \
len(self.result[num].keys()))
            print '<ol>'
            hits = self.result[num].keys()
            hits.sort()
            for hit in hits:
                print '<li><a href=\"/cgi-bin/pydicfunc.cgi?server=%s&num=%s&\
header=%s\">%s</a></li>' % (self.server, num, self.result[num][hit], hit)
            print '</ol>'

    def show_footer(self):
        print '<hr>'
        print '%s' % self.version
        print '</body>'
        print '</html>'

class NDTPDict(NDTPSearcher):
    def __init__(self, server, dicts):
        NDTPSearcher.__init__(self, server)
        self.dicts = dicts
        self.header = ''

    def search(self, header):
        self.header = header
        self.NDTP.connect(self.server)
        if len(self.dicts) == 1:
            num = self.dicts[0]
            self.NDTP.setdic(num)
            self.result[self.header] = self.NDTP.getmain(self.header)
        self.NDTP.quit()

    def show(self):
        self.show_header(self.header)
        self.show_main()
        self.show_footer()

    def show_main(self):
        prog = re.compile('&lt;(?P<link>[0-9a-zA-Z^>]+:[0-9a-zA-Z^>]+)&gt;')
        self.result[self.header] = string.replace(self.result[self.header], \
'<', '&lt;')
        self.result[self.header] = string.replace(self.result[self.header], \
'>', '&gt;')
        link = prog.search(self.result[self.header])
        while link != None:
            self.result[self.header] = string.replace\
(self.result[self.header], link.group(), \
'<a href=\"/cgi-bin/pydicfunc.cgi?server=%s&num=%s&header=%s\">%s</a>' % \
(self.server, self.dicts[0], link.group('link'), '(link)'))
            link = prog.search(self.result[self.header])
        self.result[self.header] = string.replace(self.result[self.header], \
'\n', '<br>\n')

        print '<h2>%s</h2>' % (self.dictdict[self.dicts[0]])
        print self.result[self.header]


def show_error(message):
    print '<html>'
    print '<head>'
    print '<body>'
    print '<h1>Error</h1><br>'
    print '%s<br>' % message
    print '</body>'
    print '</html>'

def main():
    try:
        dicts = []
        form = cgi.FieldStorage()
        if form.has_key('server'):
            server = form['server'].value
        if form.has_key('num'):
            for i in  string.split(form['num'].value, ','):
                dicts.append(int(i))

        if form.has_key('word'):
            word = form['word'].value
            n2 = NDTPSearcher(server)
            n2.search(word)
            n2.show()
        elif form.has_key('header'):
            header = form['header'].value
            n3 = NDTPDict(server, dicts)
            n3.search(header)
            n3.show()

    except Exception, message:
        show_error(message)

if __name__=='__main__':
    main()
